Package: rPandas
Title: Translating from R to Python's Pandas Package
Version: 0.1.4
Authors@R: 
    c(
        person("Akshat", "Maurya", email = "codingmaster902@gmail.com", role = c("aut", "cre")),
        person("Rihaan", "Satia", role = "aut"),
        person("David", "Shilane", email = "david.shilane@columbia.edu", role = "aut")
    )
Description: Provides an R interface to Python's 'pandas' library using non-standard
    evaluation. Users can write R code (e.g., rp_filter(), rp_select(), rp_mutate())
    that is translated into pandas commands and executed via 'reticulate'. Supports
    chaining, grouping, and 'summarisation', and includes a 'table_name' parameter to
    generate 'copy-pasteable' Python code. Ideal for leveraging pandas' speed and
    flexibility within the R ecosystem.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: ggplot2, knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Imports: reticulate, rlang
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-27 15:55:05 UTC; akshat
Author: Akshat Maurya [aut, cre],
  Rihaan Satia [aut],
  David Shilane [aut]
Maintainer: Akshat Maurya <codingmaster902@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 20:30:09 UTC
Built: R 4.5.2; ; 2026-05-01 17:41:29 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%notin%")
export(rp_calculate)
export(rp_check_env)
export(rp_count)
export(rp_filter)
export(rp_first_k_rows)
export(rp_last_k_rows)
export(rp_mutate)
export(rp_select)
export(rp_sort)
export(rp_summarize)
export(translate_filter)
import(reticulate)

# rPandas: The Power of Pandas, The Simplicity of R

[![R-CMD-check](https://github.com/akshat09867/rPandas/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/akshat09867/rPandas/actions/workflows/R-CMD-check.yaml)
**rPandas** bridges the gap between R and Python. It allows R users to manipulate data using familiar, `dplyr`-style syntax while leveraging the speed and efficiency of the Python `pandas` library in the background.



## Why use rPandas?

* **Best of Both Worlds:** Write code in R, but execute it in Python.
* **Zero Python Knowledge Required:** You don't need to know Python syntax to use it. rPandas handles the translation for you.
* **Learning Tool:** Curious how your R code translates to Python? Every function has a `return.as = "code"` argument that shows you the exact Pandas code being generated.
* **Seamless Integration:** Works with R data frames and integrates easily into your existing workflows using the pipe (`%>%` or `|>`).

#' Create a chained pandas command string
#'
#' @description
#' This internal function assembles a Python command string for pandas by
#' chaining together different data manipulation methods. It serves as the
#' central command generator for the package.
#'
#' @param df_name A character string for the name of the pandas DataFrame.
#' @param filter_str A string for the `.query()` method.
#' @param select_str A string for column selection (e.g., \code{['col1', 'col2']}).
#' @param sort_by_str A string for the `by` argument of `.sort_values()`.
#' @param sort_asc_str A string for the `ascending` argument of `.sort_values()`.
#' @param assign_str A string for the `.assign()` method.
#' @param drop_str A string for the `.drop()` method.
#' @param groupby_str A string for the `.groupby()` method.
#' @param agg_str A string for the `.agg()` method.
#' @param head_k Integer for `.head(k)`.
#' @param tail_k Integer for `.tail(k)`.
#' @return A character string of the complete, chained pandas command.
#' @keywords internal
create_pandas_statement <- function(df_name, filter_str= NULL, select_str= NULL, sort_by_str=NULL,sort_asc_str=NULL, assign_str=NULL,drop_str=NULL,groupby_str = NULL, agg_str = NULL, head_k=NULL, tail_k=NULL) {
  command <- df_name
  escaped_filter_str <- gsub("'", "\\\"", filter_str, fixed = TRUE)
  if (!is.null(filter_str) && nchar(filter_str) > 0) {
  command <- sprintf("%s.query('%s')", command, escaped_filter_str)
  }
  if(!is.null(select_str) && nchar(select_str)>0){
    command <- paste0(command, sprintf("[%s]",select_str))
  }
  if (!is.null(sort_by_str) && nchar(sort_by_str) > 0) {
    command <- paste0(command, sprintf('.sort_values(by=%s, ascending=%s)', sort_by_str, sort_asc_str))
  }
   
  if(!is.null(assign_str) && nchar(assign_str)>0){
    command <- paste0(command, sprintf(".assign(%s)", assign_str))
  }
 if(!is.null(drop_str) && nchar(drop_str)>0){
    command <- paste0(command, drop_str)
  }
  if (!is.null(groupby_str) && nchar(groupby_str) > 0) {
    command <- paste0(command, groupby_str)
  }

    if (!is.null(agg_str) && nchar(agg_str) > 0) {
    command <- paste0(command, agg_str)
  }

  if (!is.null(head_k) && !is.null(tail_k)) {
    stop("Only one of head_k or tail_k can be provided.")
  }
  if(!is.null(head_k) && is.numeric(head_k) && head_k>=0){
    command <- paste0(command, sprintf(".head(%s)", head_k))
  }
  if(!is.null(tail_k) && is.numeric(tail_k) && tail_k >=0){
    command <- paste0(command, sprintf(".tail(%s)", tail_k))
  }
  return(command)
}

#' rPandas: A User-Friendly R Interface to Pandas
#'
#' This package provides a set of wrapper functions that allow R users to
#' interact with Python's pandas library using familiar R syntax.
#'
#' @aliases rPandas
#' @import reticulate
#' @keywords package
"_PACKAGE"

#' Filter rows using pandas
#'
#' @description
#' Filters a data frame using an R expression translated to pandas.
#' @param .data An R data.frame or tibble.
#' @param filter_expression The filtering expression, written in R syntax.
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @param return.as What to return: "result", "code", or "all".
#' @return A `data.frame` containing the filtered rows.
#' @export
#' @examples
#' \donttest{
#' if (reticulate::py_available(initialize = TRUE) &&
#'     reticulate::py_module_available("pandas")) {
#'       rp_filter(ggplot2::diamonds, carat > 1 & price < 4000)
#' }
#' }

rp_filter <- function(.data, filter_expression, table_name = NULL,return.as = "result") {
  filter_expr <- substitute(filter_expression)

  filter_str <- translate_filter(!!filter_expr)
  command_str <- create_pandas_statement("df", filter_str = filter_str)
  

  execute_pandas_statement(r_df = .data, py_command = command_str, table_name = table_name,return.as)
}

#'Filtering columns
#' @description
#' Selects specific columns from a data frame. It captures the bare column
#' names and translates the operation into a pandas selection command.
#'
#' @param .data An R data.frame or tibble.
#' @param ... The bare column names to select (e.g., `carat, cut, price`).
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @param return.as What to return: "result", "code", or "all".
#' @return A `data.frame` containing only the selected columns.
#'
#' @export
#' @examples
#' \donttest{
#' if (reticulate::py_available(initialize = TRUE) &&
#'     reticulate::py_module_available("pandas")) {
#'   rp_select(ggplot2::diamonds, carat, cut, price)
#' }
#' }
rp_select <- function(.data, ..., table_name=NULL,return.as='result'){
  rp_check_env()
 select_c <- translate_select(...)
 command <- create_pandas_statement("df", select_str= select_c)
 execute_pandas_statement(.data, command, table_name = table_name,return.as)
}


#' Sort rows of a data frame using pandas
#'
#' @description
#' Sorts a data frame by one or more columns. It translates the R expressions
#' into a pandas `.sort_values()` command and executes it.
#'
#' @param .data An R data.frame or tibble.
#' @param ... Bare column names to sort by. Use `desc(colname)` to sort
#'   in descending order (e.g., `cut, desc(price)`).
#' @param return.as What to return: "result", "code", or "all".
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @return A `data.frame` sorted by the specified columns.
#'
#' @export
#' @examples
#' \donttest{
#' if (reticulate::py_available(initialize = TRUE) &&
#'     reticulate::py_module_available("pandas")) {
#'   
#'   # Sort by cut (ascending) and price (descending)
#'   rp_sort(ggplot2::diamonds, cut, desc(price))
#' }
#' }
rp_sort <- function(.data, ...,table_name=NULL,return.as='result') {
  rp_check_env()
  
  sort_params <- translate_sort(...)
  
  command_str <- create_pandas_statement(
    "df", 
    sort_by_str = sort_params$by, 
    sort_asc_str = sort_params$ascending
  )
  
  execute_pandas_statement(r_df = .data, py_command = command_str, table_name = table_name,return.as)
}


#' Mutate (add/modify/remove) columns using pandas
#'
#' @param .data An R data frame.
#' @param to_remove A character vector of column names to remove.
#' @param ... Named expressions for new/modified columns.
#' @param return.as Either "result", "code", or "all".
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @return A data frame or list depending on return.as.
#' @export
rp_mutate <- function(.data, to_remove = NULL, ...,table_name=NULL, return.as = "result") {
  rp_check_env()
  
  trans <- translate_mutate(to_remove, ...)
    command_str <- create_pandas_statement(
    df_name = "df",
    assign_str = if (nzchar(trans$assign_str)) trans$assign_str else NULL,
    drop_str = trans$drop_str
  )
  
  execute_pandas_statement(r_df = .data, py_command = command_str, table_name = table_name, return.as = return.as)
}

#' Summarize data using pandas
#'
#' @description
#' Aggregates a data frame by one or more groups, applying summary functions.
#' It translates R's `dplyr::summarise` syntax into a pandas `.groupby().agg()`
#' command.
#'
#' @param .data An R data.frame or tibble.
#' @param ... Named summary expressions (e.g., `avg_price = mean(price)`).
#'   Supports `mean`, `median`, `sd`, `var`, `min`, `max`, `sum`, and `n()`.
#' @param .by A bare column name or `c(col1, col2)` to group by.
#' @param return.as What to return: "result", "code", or "all".
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @return A `data.frame` with the summarized and grouped data.
#'
#' @export
#' @examples
#' \donttest{
#' if (reticulate::py_available(initialize = TRUE) &&
#'     reticulate::py_module_available("pandas")) {
#'   
#'   # Summarize by one group
#'   rp_summarize(ggplot2::diamonds, 
#'                avg_price = mean(price), 
#'                .by = cut)
#'   
#'   # Summarize by multiple groups and multiple functions
#'   rp_summarize(ggplot2::diamonds, 
#'                avg_price = mean(price), 
#'                count = n(),
#'                .by = c(cut, color))
#' }
#' }
rp_summarize <- function(.data, ..., .by = NULL,table_name=NULL,return.as='result') {
  rp_check_env()
  
  groupby_str <- translate_groupby(rlang::enquo(.by))
  
  agg_str <- translate_summarize(...)
  
  command_str <- create_pandas_statement(
    "df", 
    groupby_str = groupby_str,
    agg_str = agg_str
  )
  
  execute_pandas_statement(r_df = .data, py_command = command_str, table_name = table_name,return.as)
}



#' Apply multiple summary functions to multiple columns
#'
#' @description
#' Applies a list of summary functions to a list of columns, after
#' optionally grouping the data.
#'
#' @param .data An R data.frame.
#' @param ... Bare column names to summarize (e.g., `price, carat`).
#' @param the.functions A character vector of R function names
#'   (e.g., `c("mean", "sd")`). Supports "mean", "median", "sd",
#'   "var", "min", "max", "sum".
#' @param .by A bare column name or `c(col1, col2)` to group by.
#' @param return.as What to return: "result", "code", or "all".
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @return A `data.frame` with the summarized and grouped data.
#' @export
#' @examples
#' \donttest{
#' if (reticulate::py_available(initialize = TRUE) &&
#'     reticulate::py_module_available("pandas")) {
#'
#'   rp_calculate(
#'     ggplot2::diamonds,
#'     price, carat,
#'     the.functions = c("mean", "sd"),
#'     .by = cut
#'   )
#' }
#' }
rp_calculate <- function(.data, ..., the.functions, .by = NULL,table_name=NULL,return.as='result') {
  rp_check_env()
  
  groupby_str <- translate_groupby(rlang::enquo(.by))
  
  variable_exprs <- rlang::enquos(...)
  if (rlang::is_empty(variable_exprs)) {
    stop("At least one variable (...) must be provided.", call. = FALSE)
  }
  if (missing(the.functions) || !is.character(the.functions)) {
    stop("`the.functions` must be a character vector.", call. = FALSE)
  }
  
  agg_str <- translate_calculate(variable_exprs, the.functions)
  
  command_str <- create_pandas_statement(
    "df", 
    groupby_str = groupby_str,
    agg_str = agg_str
  )
  
  execute_pandas_statement(r_df = .data, py_command = command_str, table_name = table_name, return.as)
}


#' Extract the first k rows of a data frame
#'
#' This function returns the first k rows of the data frame. If grouping variables
#' are provided via `.by`, it returns the first k rows within each group.
#'
#' @param .data An R data frame (or tibble) to be processed.
#' @param k An integer specifying the number of rows to return. If `.by` is used,
#'   returns up to k rows per group.
#' @param .by Optional grouping variables. Can be one or more unquoted column names.
#'   When provided, the operation is performed on each group separately.
#' @param return.as One of `"result"`, `"code"`, or `"all"`.
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @return Depending on `return.as`: a data frame, a character string, or a list.
#' @export
rp_first_k_rows <- function(.data, k, .by=NULL,table_name=NULL, return.as= "result"){
    groupby_str <- translate_groupby(rlang::enquo(.by))
    f_expr <- create_pandas_statement("df", groupby_str = groupby_str, head_k = k)
  execute_pandas_statement(r_df = .data, py_command = f_expr, table_name = table_name, return.as)
}


#' Extract the last k rows of a data frame
#'
#' This function returns the last k rows of the data frame. If grouping variables
#' are provided via `.by`, it returns the last k rows within each group.
#'
#' @param .data An R data frame (or tibble) to be processed.
#' @param k An integer specifying the number of rows to return. If `.by` is used,
#'   returns up to k rows per group.
#' @param .by Optional grouping variables. Can be one or more unquoted column names.
#'   When provided, the operation is performed on each group separately.
#' @param return.as One of `"result"`, `"code"`, or `"all"`.
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @return Depending on `return.as`: a data frame, a character string, or a list.
#' @export
rp_last_k_rows <- function(.data, k, .by=NULL,table_name=NULL, return.as="result"){
  groupby_str <- translate_groupby(rlang::enquo(.by))
  f_expr <- create_pandas_statement("df", groupby_str = groupby_str, tail_k = k)
  execute_pandas_statement(r_df = .data, py_command = f_expr, table_name = table_name, return.as)
}

#' Count rows in a data frame, optionally by groups
#'
#' This function returns the number of rows in a data frame. When grouping
#' variables are provided via `.by`, it returns the row counts for each group.
#'
#' @param .data An R data frame (or tibble) to be processed.
#' @param .by Optional grouping variables. Can be one or more unquoted column names
#'   (e.g., `cut` or `c(cut, color)`). When provided, counts are computed per group.
#' @param return.as One of `"result"`, `"code"`, or `"all"`.
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @return A data frame with one column `"n"` (total row count) if `.by = NULL`,
#'     or a data frame with the grouping columns and a column `"n"` (per‑group counts).
#' @export
rp_count <- function(.data, .by = NULL,table_name=NULL, return.as = "result"){
  by_quo <- rlang::enquo(.by)
  
  if (rlang::quo_is_missing(by_quo) || rlang::quo_is_null(by_quo)) {
    py_cmd <- "pd.DataFrame({'n': [len(df)]})"
    
  } else {
   by_expr <- rlang::quo_get_expr(by_quo)
        groupby_str <- translate_groupby(by_expr)
        py_cmd <- paste0("df", groupby_str, ".size().rename(columns={'size': 'n'})")
  }
  
  execute_pandas_statement(r_df = .data, py_command = py_cmd, table_name = table_name, return.as = return.as)
}

#' Executes a pandas command string on an R data frame.
#'
#' This is the core execution engine. It explicitly injects an R data frame
#' into the Python session, runs a command, retrieves the result, and cleans up.
#'
#' @param r_df An R data.frame.
#' @param py_command A character string of Python code using 'df' as a placeholder.
#' @param table_name An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).
#' @param return.as What to return: "result", "code", or "all".
#' @return The result of the execution.
#' @keywords internal
execute_pandas_statement <- function(r_df, py_command, table_name = table_name, return.as = "result") {
  rp_check_env()
  if (!return.as %in% c("result", "code", "all")) {
    stop("return.as must be 'result', 'code', or 'all'.", call. = FALSE)
  }
  
  reticulate::py_run_string("import pandas as pd")
  
  py_df_name <- "rpandas_df_in"
  py[[py_df_name]] <- r_df
  
  full_py_command <- gsub("\\bdf\\b", py_df_name, py_command, perl = TRUE)
    display_code <- full_py_command
  if (!is.null(table_name) && is.character(table_name)) {
    display_code <- gsub("rpandas_df_in", table_name, display_code, fixed = TRUE)
  } else {
    display_code <- gsub("rpandas_df_in", "df", display_code, fixed = TRUE)
  }
  if (return.as == "code") {
    reticulate::py_run_string(paste("del", py_df_name))
    return(display_code)
  }
  
  py_script <- sprintf("
import pandas as pd
import numpy as np

rpandas_df_out = %s

if isinstance(rpandas_df_out, pd.DataFrame) and isinstance(rpandas_df_out.columns, pd.MultiIndex):
    rpandas_df_out.columns = ['.'.join(col).strip('.') for col in rpandas_df_out.columns.values]

if isinstance(rpandas_df_out.index, pd.MultiIndex):
    rpandas_df_out = rpandas_df_out.reset_index()
", full_py_command)

  reticulate::py_run_string(py_script)
  reticulate::py_run_string(paste("del", py_df_name))
  
  result_from_py <- reticulate::py$rpandas_df_out
  
  result_in_r <- tryCatch(
    reticulate::py_to_r(result_from_py),
    error = function(e) result_from_py  
  )
  
  if (inherits(result_in_r, "python.builtin.object")) {
    reticulate::py_run_string("rpandas_dict = rpandas_df_out.to_dict(orient='list')")
    dict_from_py <- reticulate::py$rpandas_dict
    result_in_r <- as.data.frame(dict_from_py, stringsAsFactors = FALSE)
    reticulate::py_run_string("del rpandas_dict")
  }
  
  reticulate::py_run_string("del rpandas_df_out")
  
  if (return.as == "result") {
    return(result_in_r)
  }
  
  list(result = result_in_r, code = display_code)
}

library(testthat)

# Load the function (devtools::load_all())

# Test 1: AND operator (&)
translate_filter(carat > 1 & cut == "Ideal")
#> [1] "(carat > 1) and (cut == 'Ideal')"

# Test 2: OR operator (|)
translate_filter(clarity == "VVS1" | clarity == "VVS2")
#> [1] "(clarity == 'VVS1') or (clarity == 'VVS2')"

# Test 3: NOT operator (!)
translate_filter(!(cut == "Ideal"))
#> [1] "(not (cut == 'Ideal'))"

# Test 4: %in% operator
allowed_colors <- c("G", "H", "I")
translate_filter(color %in% allowed_colors)
#> [1] "(color in @allowed_colors)"

# Test 5: %notin% operator (newly added)
# Define a placeholder function for the test
`%notin%` <- function(x, y) !x %in% y
translate_filter(color %notin% c("D", "E", "F"))
#> [1] "(color not in ['D', 'F', 'F'])"

# Test 6: Complex nested expression
translate_filter(
  (carat > 0.9 & price < 10000) | (cut == "Premium" & !(color %in% c("J", "I")))
)
#> [1] "((carat > 0.9) and (price < 10000)) or ((cut == 'Premium') and (not (color in ['J', 'I'])))"

#' Check for rPandas dependencies and provide diagnostics
#'
#' @description
#' This function checks if the user's system is correctly configured with Python
#' and the pandas library. If dependencies are missing, it stops with a
#' detailed diagnostic report and actionable instructions (only in interactive sessions).
#' In non‑interactive contexts (e.g., CRAN checks), it issues a warning and returns FALSE.
#'
#' @export
#' @return Invisibly returns `TRUE` if all checks pass, otherwise `FALSE`.
rp_check_env <- function() {
  # Helper to produce error or warning depending on context
  notify <- function(msg, is_error = FALSE) {
    if (interactive() && is_error) {
      stop(msg, call. = FALSE)
    } else {
      warning(msg, call. = FALSE, immediate. = TRUE)
      return(FALSE)
    }
  }

  if (!reticulate::py_available(initialize = TRUE)) {
    msg <- paste0(
      "Python could not be found by reticulate.\n\n",
      "--- How to Fix ---\n",
      "Option 1 (recommended): Install Miniconda (self-contained Python)\n",
      "  Run in R: reticulate::install_miniconda()\n\n",
      "Option 2: Use an existing Python installation\n",
      "  Run in R before loading rPandas:\n",
      '  reticulate::use_python("/path/to/your/python", required = TRUE)\n',
      "  Find your Python path with: Sys.which('python')\n\n",
      "After fixing, restart R and try again.\n",
      "------------------------------"
    )
    return(notify(msg, is_error = TRUE))
  }

  if (!reticulate::py_module_available("pandas")) {
    config <- reticulate::py_config()
    python_path <- config$python

    msg <- paste0(
      "The 'pandas' Python library was not found in the current environment.\n\n",
      "--- rPandas System Diagnosis ---\n",
      "rPandas is currently configured to use this Python installation:\n",
      "> ", python_path, "\n\n",
      "This environment does NOT contain the 'pandas' library.\n"
    )

    conda_envs <- tryCatch(reticulate::conda_list(), error = function(e) NULL)
    if (!is.null(conda_envs) && nrow(conda_envs) > 0) {
      msg <- paste0(msg,
        "\n--- Available Conda Environments ---\n",
        paste0(conda_envs$name, " (", conda_envs$python, ")", collapse = "\n"),
        "\n\n"
      )
      msg <- paste0(msg,
        "Try switching to the first environment:\n",
        '  reticulate::use_python(python = conda_list()$python[1], required = TRUE)\n',
        "Then restart R and reload rPandas.\n\n"
      )
    } else {
      msg <- paste0(msg,
        "\nNo conda environments found. You can create one with pandas:\n",
        "  reticulate::conda_create('r-reticulate')\n",
        "  reticulate::conda_install('r-reticulate', 'pandas')\n",
        "  reticulate::use_condaenv('r-reticulate', required = TRUE)\n\n"
      )
    }

    msg <- paste0(msg,
      "--- How to Fix (quick) ---\n",
      "To install pandas into the current Python environment, run:\n",
      '  reticulate::py_install("pandas")\n\n',
      "If the path above is incorrect, restart R and set the correct Python path *before* loading rPandas:\n",
      '  reticulate::use_python("/correct/path", required = TRUE)\n',
      "------------------------------"
    )
    return(notify(msg, is_error = TRUE))
  }

  invisible(TRUE)
}

#' Translate an R filter expression into a Python query string
#'
#' @description
#' Capture a bare R expression and translate it to a Python-compatible string
#' suitable for use with `pandas.DataFrame.query()`.
#' @param expr A bare R expression (e.g., `carat > 2 & cut == "Ideal"`).
#' @return A character string of the translated Python query.
#' @export
#' @examples
#' translate_filter(carat > 2 & cut == "Ideal")
#' # -> "(carat > 1) and (cut == 'Ideal')"
translate_filter <- function(expr) {
  expr_quo <- rlang::enquo(expr)
  expr_env <- rlang::get_env(expr_quo)
  expr_body <- rlang::get_expr(expr_quo)

  out <- translate_filter_recursive(expr_body, env = expr_env)
  
  out <- sub("^\\((.*)\\)$", "\\1", out)
  return(out)
}


#' Recursive helper to translate R expressions
#' @param expr_body A language object (call, symbol, or atomic).
#' @return A character string.
#' @keywords internal
translate_filter_recursive <- function(expr_body, env) {
  if (is.symbol(expr_body)) {
    return(as.character(expr_body))
  }
  if (is.atomic(expr_body)) {
    if (is.logical(expr_body) && length(expr_body) == 1) {
      if (is.na(expr_body)) return("None")
      return(ifelse(expr_body, "True", "False"))
    }
    if (is.character(expr_body)) {
      esc <- gsub("'", "\\\\'", expr_body, fixed = TRUE)
      if (length(esc) > 1) {
        return(sprintf("['%s']", paste(esc, collapse = "', '")))
      }
      return(sprintf("'%s'", esc))
    }
    if (is.numeric(expr_body)) {
      if (length(expr_body) > 1) {
        return(sprintf("[%s]", paste(expr_body, collapse = ", ")))
      }
      return(as.character(expr_body))
    }
    return(as.character(expr_body))
  }

  if (is.call(expr_body)) {
    op <- as.character(expr_body[[1]])
    args <- as.list(expr_body)[-1]
    
    tr <- function(x) translate_filter_recursive(x, env = env)

    if (op == "(") {
      return(tr(args[[1]]))
    }
    
    if (op == "c") {
      pieces <- vapply(args, tr, FUN.VALUE = character(1))
      return(paste0("[", paste(pieces, collapse = ", "), "]"))
    }
    
    if (op %in% c("&", "&&")) return(paste0("(", tr(args[[1]]), " and ", tr(args[[2]]), ")"))
    if (op %in% c("|", "||")) return(paste0("(", tr(args[[1]]), " or ", tr(args[[2]]), ")"))
    if (op == "!") return(paste0("(not ", tr(args[[1]]), ")"))
    if (op == "is.na") return(paste0("(", tr(args[[1]]), ".isna())"))

    if (op %in% c("%in%", "%notin%")) {
      py_op <- ifelse(op == "%in%", "in", "not in")
      lhs <- tr(args[[1]])
      
      rhs_val <- eval(args[[2]], envir = env)
      
      rhs_str <- tr(rhs_val)
      
      return(paste0("(", lhs, " ", py_op, " ", rhs_str, ")"))
    }

    py_op <- switch(op, "%/%" = "//", "%%"  = "%", op) 
    
    if (py_op %in% c("==", "!=", ">", ">=", "<", "<=", "+", "-", "*", "/", "//", "%")) {
      return(paste0("(", tr(args[[1]]), " ", py_op, " ", tr(args[[2]]), ")"))
    }

    translated_args <- vapply(args, tr, character(1))
    return(paste0(op, "(", paste(translated_args, collapse = ", "), ")"))
  }

  stop("Unrecognized expression type in translate_filter_recursive()")
}

#' "Not In" Operator
#'
#' @description
#' Provides the opposite of the standard R `%in%` operator.
#' @param x Vector of values to be matched.
#' @param y Vector of values to be matched against.
#' @return A logical vector.
#' @export
#' @examples
#' "a" %notin% c("b", "c")
`%notin%` <- function(x, y) {
  !(x %in% y)
}


#' Translate captured column names into a Python list string
#'
#' @param ... Bare column names (captured by `rlang::enquos`).
#' @return A character string formatted as a Python list.
#' @keywords internal
translate_select <- function(...) {
  exprs <- rlang::enquos(...)

  col_names <- vapply(exprs, function(expr) {
    text <- rlang::expr_text(expr)
      gsub("^~", "", text)  
  }, character(1))

  py_list_str <- sprintf("['%s']", paste(col_names, collapse = "', '"))
  
  return(py_list_str)
}

#' Translate captured sort expressions into Python .sort_values() arguments
#'
#' @param ... Bare column names or `desc(colname)` (captured by `rlang::enquos`).
#' @return A list with two elements:
#'   - `$by`: A string for the `by` argument (e.g., \code{['cut', 'price']})
#'   - `$ascending`: A string for the `ascending` argument (e.g., \code{[True, False]})
#' @keywords internal
translate_sort <- function(...) {
  exprs <- rlang::enquos(...)

  cols <- character()
  asc  <- logical()

  for (q in exprs) {
    e <- rlang::get_expr(q)

    if (is.call(e) && rlang::call_name(e) == "desc") {
      cols <- c(cols, rlang::as_string(e[[2]]))
      asc  <- c(asc, FALSE)
    } else {
      cols <- c(cols, rlang::as_string(e))
      asc  <- c(asc, TRUE)
    }
  }

  by_str  <- sprintf("['%s']", paste(cols, collapse = "', '"))
  asc_py  <- ifelse(asc, "True", "False")
  asc_str <- sprintf("[%s]", paste(asc_py, collapse = ", "))

  return(list(by = by_str, ascending = asc_str))
}

#' Translate R mutate expressions to pandas assign/drop strings
#'
#' @param to_remove Character vector of column names to drop.
#' @param ... Named R expressions.
#' @return A list with components assign_str and drop_str.
#' @keywords internal
translate_mutate <- function(to_remove = NULL, ...) {
  exprs <- rlang::enquos(...)
    if (length(exprs) > 0) {
    nm <- names(exprs)
    if (is.null(nm) || any(nm == "")) {
      stop("All arguments to rp_mutate() must be named (e.g., new_col = expression).",
           call. = FALSE)
    }
  }
  
  assign_pieces <- character()
  if (length(exprs) > 0) {
   assign_pieces <- vapply(seq_along(exprs), function(i) {
      q <- exprs[[i]]
      col_name <- names(exprs)[i]
      translated <- translate_assign_recursive(rlang::get_expr(q))
      paste0(col_name, " = lambda x: ", translated)
    }, FUN.VALUE = character(1))
  }
  
  drop_str <- NULL
  if (length(to_remove) > 0) {
    if (!is.character(to_remove)) {
      stop("`to_remove` must be a character vector of column names.", call. = FALSE)
    }
    quoted <- paste0('"', to_remove, '"')
    drop_str <- sprintf(".drop(columns = [%s])", paste(quoted, collapse = ", "))
  }
  
  list(
    assign_str = paste(assign_pieces, collapse = ", "),
    drop_str = drop_str
  )
}

#' Recursively translate an R expression for a pandas .assign() lambda
#'
#' @param expr_body A language object (call, symbol, or atomic).
#' @return A character string of the translated Python expression.
#' @keywords internal
translate_assign_recursive <- function(expr_body) {
  if (is.symbol(expr_body)) {
    return(sprintf("x['%s']", as.character(expr_body)))
  }

  if (is.atomic(expr_body)) {
    if (is.logical(expr_body) && length(expr_body) == 1) {
      if (is.na(expr_body)) return("None")
      return(ifelse(expr_body, "True", "False"))
    }
    if (is.character(expr_body)) {
      esc <- gsub("'", "\\\\'", expr_body, fixed = TRUE)
      return(sprintf("'%s'", esc))
    }
    return(as.character(expr_body))
  }

  if (is.call(expr_body)) {
    op <- as.character(expr_body[[1]])
    args <- as.list(expr_body)[-1]
    
    tr <- function(x) translate_assign_recursive(x)
  if (op == "(") {
      return(tr(args[[1]]))
    }
    
    
    if (op %in% c("&", "&&")) return(paste0("(", tr(args[[1]]), " and ", tr(args[[2]]), ")"))
    if (op %in% c("|", "||")) return(paste0("(", tr(args[[1]]), " or ", tr(args[[2]]), ")"))
    if (op == "!") return(paste0("not (", tr(args[[1]]), ")"))

    if (op == "is.na") return(paste0(tr(args[[1]]), ".isna()"))
    
    py_op <- switch(op,
                    "%/%" = "//",
                    "%%"  = "%",
                    op)
    
    if (py_op %in% c("==", "!=", ">", ">=", "<", "<=", "+", "-", "*", "/", "//", "%")) {
      return(paste0("(", tr(args[[1]]), " ", py_op, " ", tr(args[[2]]), ")"))
    }

    translated_args <- vapply(args, tr, character(1))
    if (grepl("^\\((.*)\\)$", translated_args)) {
  translated_args <- gsub("^\\((.*)\\)$", "\\1", translated_args)
}
    return(paste0(op, "(", paste(translated_args, collapse = ", "), ")"))
  }

  stop("Unrecognized expression type in translate_assign_recursive()")
}




#' Translate a .by argument into a pandas .groupby() string
#'
#' @param by_expr An enquosured `.by` argument.
#' @return A string for the `.groupby(..., as_index=False)` method,
#'   or NULL if the argument is empty.
#' @keywords internal
translate_groupby <- function(by_expr) {
  expr_body <- rlang::get_expr(by_expr)
  if (rlang::is_null(expr_body)) return(NULL)
  
  col_names <- character()
  if (is.call(expr_body) && rlang::call_name(expr_body) == "c") {
    args <- rlang::call_args(expr_body)
    col_names <- vapply(args, function(arg) {
      txt <- rlang::expr_text(arg)
      gsub('^"|"$', '', txt)
    }, character(1))
  } else {
    txt <- rlang::expr_text(expr_body)
    col_names <- gsub('^"|"$', '', txt)
  }
  
  py_list_str <- sprintf("['%s']", paste(col_names, collapse = "', '"))
  sprintf(".groupby(%s, as_index=False, observed=True)", py_list_str)
}


#' Translate named R expressions for .agg()
#'
#' @description
#' Translates R's `new = func(old)` syntax into pandas' named aggregation
#' syntax `new = ('old', 'func')`.
#'
#' @param ... Named R expressions (e.g., `avg_price = mean(price)`).
#' @return A string for the `.agg()` method.
#' @keywords internal
translate_summarize <- function(...) {
  exprs <- rlang::enquos(...)
  
  if (rlang::is_empty(exprs)) {
    stop("No summary functions provided.", call. = FALSE)
  }
  if (is.null(names(exprs)) || any(names(exprs) == "")) {
    stop("All arguments to rp_summarize() must be named.", call. = FALSE)
  }
  
  new_col_names <- names(exprs)
  agg_pieces <- character(length(exprs))
  
  for (i in seq_along(exprs)) {
    new_name <- new_col_names[i]
    expr_body <- rlang::get_expr(exprs[[i]])
    
    if (!is.call(expr_body)) {
      stop("Summarize expressions must be function calls (e.g., mean(price)).", call. = FALSE)
    }
    
    r_func_name <- rlang::call_name(expr_body)

    
    if (r_func_name == "n") {
      py_func_name <- "size"

      r_col_name <- "price" 
    } else {
      call_args <- rlang::call_args(expr_body)
      
      if (rlang::is_empty(call_args)) {
        stop(paste("Function", r_func_name, "needs a column argument."), call. = FALSE)
      }
      
      r_col_name <- rlang::expr_text(call_args[[1]])
      
      py_func_name <- switch(r_func_name,
        "mean" = "mean",
        "median" = "median",
        "sd" = "std",
        "var" = "var",
        "min" = "min",
        "max" = "max",
        "sum" = "sum",
        stop(paste("Unknown summary function:", r_func_name), call. = FALSE)
      )
    }
    agg_pieces[i] <- sprintf("%s = ('%s', '%s')", new_name, r_col_name, py_func_name)
  }
  
  sprintf(".agg(%s)", paste(agg_pieces, collapse = ", "))
}



#' Translate R function/column names into a pandas agg dictionary
#'
#' @description
#' Translates R's `the.variables` and `the.functions` into pandas'
#' dictionary-based `.agg()` syntax.
#'
#' @param variable_exprs A list of enquosured variable names.
#' @param function_names A character vector of R function names.
#' @return A string for the `.agg()` method (e.g., \code{.agg({'col1': ['mean', 'std']})}).
#' @keywords internal
translate_calculate <- function(variable_exprs, function_names) {
  
  map_r_to_py_func <- function(r_func_name) {
    py_func_name <- switch(r_func_name,
      "mean" = "mean",
      "median" = "median",
      "sd" = "std",
      "var" = "var",
      "min" = "min",
      "max" = "max",
      "sum" = "sum",
      stop(paste("Unknown summary function:", r_func_name), call. = FALSE)
    )
    return(py_func_name)
  }
    cols <- vapply(variable_exprs, function(expr) {
    text <- rlang::expr_text(expr)
      gsub("^~", "", text)  
  }, character(1))
  
  py_funcs <- vapply(function_names, map_r_to_py_func, character(1))
  py_funcs_str <- sprintf("['%s']", paste(py_funcs, collapse = "', '"))
  
  agg_pieces <- vapply(cols, function(col) {
    sprintf("'%s': %s", col, py_funcs_str)
  }, character(1))
  
  dict_str <- paste(agg_pieces, collapse = ", ")
  sprintf(".agg({%s})", dict_str)
}

---
title: "Introduction-to-rPandas"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{introduction-to-rPandas}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(eval=FALSE,
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, include=FALSE}
library(rPandas)
```


## Introduction
rPandas is an R package designed to serve as a translation layer, allowing R users to leverage the power and speed of the Python pandas library without ever leaving their R session.

For R users familiar with dplyr or data.table, the rPandas syntax will feel natural. The package provides a set of R functions (e.g., rp_filter(), rp_select()) that:

Capture R code (like carat > 1).

Translate it into an equivalent Python/pandas command string.

Execute the Python code in the background via the reticulate package.

Return the resulting data.frame back to your R session.

This allows you to write R code while the data processing is handled by Python's pandas library.

## Setup and Installation
### Python Dependencies
rPandas depends on a working Python installation and the pandas library. The reticulate package handles the R‑to‑Python connection.

### Quick installation (default environment)
You can install pandas into a dedicated R environment by running:

```{r}
reticulate::py_install("pandas")
```
#### Troubleshooting the Python environment

If you encounter issues (e.g., “pandas not found”), rPandas provides a built‑in health check:

```{r}
# Run this if you have connection issues
rp_check_env()
```

The health check prints the Python path reticulate is using and whether `pandas` is installed.

##### Finding and selecting the correct Python environment

You can list all available conda environments (if you use Conda) with:

```{r}
reticulate::conda_list()
```

Once you identify the desired environment (e.g., the first one), you can tell reticulate to use it **before loading rPandas**:

```{r}
# Replace with the path from conda_list()
reticulate::use_python(python = reticulate::conda_list()$python[1], required = TRUE)

# Or, if you prefer to use a conda environment by name:
reticulate::use_condaenv("your_environment_name", required = TRUE)
```

For system Python installations, simply provide the path to the Python binary:

```{r}
reticulate::use_python("/usr/local/bin/python3", required = TRUE)
```

After setting the environment, rerun `rp_check_env()` to confirm everything is ready. For more detailed guidance, see the [reticulate documentation](https://rstudio.github.io/reticulate/).

> **Note:** The code chunks above are not executed when building the vignette (they are meant to be run interactively by the user).

## The Core Verbs
All examples will use the diamonds dataset from the ggplot2 package.

```{r eval=TRUE}
# Make sure ggplot2 is installed to access the data
data(diamonds, package = "ggplot2")
head(diamonds)

```
**1. rp_filter():**
This verb filters rows based on a logical expression, similar to dplyr::filter().
It supports common logical operators: & (AND), | (OR), ! (NOT), %in%, and %notin%

```{r} 
# Simple condition
v1 <- rp_filter(diamonds, carat > 1)
print(head(v1))
#>   carat       cut color clarity depth table price    x    y    z
#> 1  1.17 Very Good     J      I1  60.2    61  2774 6.83 6.90 4.13
#> 2  1.01   Premium     F      I1  61.8    60  2781 6.39 6.36 3.94
#> 3  1.01      Fair     E      I1  64.5    58  2788 6.29 6.21 4.03
#> 4  1.01   Premium     H     SI2  62.7    59  2788 6.31 6.22 3.93
#> 5  1.05 Very Good     J     SI2  63.2    56  2789 6.49 6.45 4.09
#> 6  1.05      Fair     J     SI2  65.8    59  2789 6.41 6.27 4.18

# AND: multiple conditions
v2 <- rp_filter(diamonds, carat > 1 & cut == "Ideal")
print(head(v2))
#>   carat   cut color clarity depth table price    x    y    z
#> 1  1.01 Ideal     I      I1  61.5    57  2844 6.45 6.46 3.97
#> 2  1.02 Ideal     H     SI2  61.6    55  2856 6.49 6.43 3.98
#> 3  1.02 Ideal     I      I1  61.7    56  2872 6.44 6.49 3.99
#> 4  1.02 Ideal     J     SI2  60.3    54  2879 6.53 6.50 3.93
#> 5  1.01 Ideal     I      I1  61.5    57  2896 6.46 6.45 3.97
#> 6  1.02 Ideal     I      I1  61.7    56  2925 6.49 6.44 3.99

# OR: use | (pipe)
v3 <- rp_filter(diamonds, color == "D" | color == "E")
print(head(v3))
#>   carat     cut color clarity depth table price    x    y    z
#> 1  0.23   Ideal     E     SI2  61.5    55   326 3.95 3.98 2.43
#> 2  0.21 Premium     E     SI1  59.8    61   326 3.89 3.84 2.31
#> 3  0.23    Good     E     VS1  56.9    65   327 4.05 4.07 2.31
#> 4  0.22    Fair     E     VS2  65.1    61   337 3.87 3.78 2.49
#> 5  0.20 Premium     E     SI2  60.2    62   345 3.79 3.75 2.27
#> 6  0.32 Premium     E      I1  60.9    58   345 4.38 4.42 2.68


# NOT: use !
v4 <- rp_filter(diamonds, !(price > 10000))
print(head(v4))
#>   carat       cut color clarity depth table price    x    y    z
#> 1  0.23     Ideal     E     SI2  61.5    55   326 3.95 3.98 2.43
#> 2  0.21   Premium     E     SI1  59.8    61   326 3.89 3.84 2.31
#> 3  0.23      Good     E     VS1  56.9    65   327 4.05 4.07 2.31
#> 4  0.29   Premium     I     VS2  62.4    58   334 4.20 4.23 2.63
#> 5  0.31      Good     J     SI2  63.3    58   335 4.34 4.35 2.75
#> 6  0.24 Very Good     J    VVS2  62.8    57   336 3.94 3.96 2.48


# %in% operator
v5 <- rp_filter(diamonds, color %in% c("D", "E", "F"))
print(head(v5))
#>   carat     cut color clarity depth table price    x    y    z
#> 1  0.23   Ideal     E     SI2  61.5    55   326 3.95 3.98 2.43
#> 2  0.21 Premium     E     SI1  59.8    61   326 3.89 3.84 2.31
#> 3  0.23    Good     E     VS1  56.9    65   327 4.05 4.07 2.31
#> 4  0.22    Fair     E     VS2  65.1    61   337 3.87 3.78 2.49
#> 5  0.22 Premium     F     SI1  60.4    61   342 3.88 3.84 2.33
#> 6  0.20 Premium     E     SI2  60.2    62   345 3.79 3.75 2.27

# %notin% (if implemented)
v6 <- rp_filter(diamonds, color %notin% c("D", "E", "F"))
print(head(v6))
#>   carat       cut color clarity depth table price    x    y    z
#> 1  0.29   Premium     I     VS2  62.4    58   334 4.20 4.23 2.63
#> 2  0.31      Good     J     SI2  63.3    58   335 4.34 4.35 2.75
#> 3  0.24 Very Good     J    VVS2  62.8    57   336 3.94 3.96 2.48
#> 4  0.24 Very Good     I    VVS1  62.3    57   336 3.95 3.98 2.47
#> 5  0.26 Very Good     H     SI1  61.9    55   337 4.07 4.11 2.53
#> 6  0.23 Very Good     H     VS1  59.4    61   338 4.00 4.05 2.39
```

**2. rp_select():**
This verb selects specific columns by name, similar to dplyr::select().

```{r}
# Select three columns
v4 <- rp_select(diamonds, carat, cut, price)
print(head(v4))
#>   carat       cut price
#> 1  0.23     Ideal   326
#> 2  0.21   Premium   326
#> 3  0.23      Good   327
#> 4  0.29   Premium   334
#> 5  0.31      Good   335
#> 6  0.24 Very Good   336
```

**3. rp_sort():**
This verb sorts the data frame by one or more columns, similar to dplyr::arrange().
```{r}
# Sort by price (ascending by default)
v8 <- rp_sort(diamonds, price)
print(head(v8))
#>   carat       cut color clarity depth table price    x    y    z
#> 1  0.23     Ideal     E     SI2  61.5    55   326 3.95 3.98 2.43
#> 2  0.21   Premium     E     SI1  59.8    61   326 3.89 3.84 2.31
#> 3  0.23      Good     E     VS1  56.9    65   327 4.05 4.07 2.31
#> 4  0.29   Premium     I     VS2  62.4    58   334 4.20 4.23 2.63
#> 5  0.31      Good     J     SI2  63.3    58   335 4.34 4.35 2.75
#> 6  0.24 Very Good     I    VVS1  62.3    57   336 3.95 3.98 2.47

# Use desc() to sort in descending order
v9 <- rp_sort(diamonds, desc(price))
print(head(v9))
#>   carat       cut color clarity depth table price    x    y    z
#> 1  2.29   Premium     I     VS2  60.8    60 18823 8.50 8.47 5.16
#> 2  2.00 Very Good     G     SI1  63.5    56 18818 7.90 7.97 5.04
#> 3  1.51     Ideal     G      IF  61.7    55 18806 7.37 7.41 4.56
#> 4  2.07     Ideal     G     SI2  62.5    55 18804 8.20 8.13 5.11
#> 5  2.00 Very Good     H     SI1  62.8    57 18803 7.95 8.00 5.01
#> 6  2.29   Premium     I     SI1  61.8    59 18797 8.52 8.45 5.24

# Sort by multiple columns
v10 <- rp_sort(diamonds, cut, desc(price))
print(head(v10))
#>   carat  cut color clarity depth table price     x     y    z
#> 1  2.01 Fair     G     SI1  70.6    64 18574  7.43  6.64 4.69
#> 2  2.02 Fair     H     VS2  64.5    57 18565  8.00  7.95 5.14
#> 3  4.50 Fair     J      I1  65.8    58 18531 10.23 10.16 6.72
#> 4  2.00 Fair     G     VS2  67.6    58 18515  7.65  7.61 5.16
#> 5  2.51 Fair     H     SI2  64.7    57 18308  8.44  8.50 5.48
#> 6  3.01 Fair     I     SI2  65.8    56 18242  8.99  8.94 5.90
```


**4. rp_mutate():**
This verb creates new columns or modifies existing ones, similar to dplyr::mutate().
You can also remove columns using the to_remove argument.
```{r}

# Create a new column
v11 <- rp_mutate(diamonds, price_per_carat = price / carat)
print(head(v11))
#>   carat       cut color clarity depth table price    x    y    z
#> 1  0.23     Ideal     E     SI2  61.5    55   326 3.95 3.98 2.43
#> 2  0.21   Premium     E     SI1  59.8    61   326 3.89 3.84 2.31
#> 3  0.23      Good     E     VS1  56.9    65   327 4.05 4.07 2.31
#> 4  0.29   Premium     I     VS2  62.4    58   334 4.20 4.23 2.63
#> 5  0.31      Good     J     SI2  63.3    58   335 4.34 4.35 2.75
#> 6  0.24 Very Good     J    VVS2  62.8    57   336 3.94 3.96 2.48
#>   price_per_carat
#> 1        1417.391
#> 2        1552.381
#> 3        1421.739
#> 4        1151.724
#> 5        1080.645
#> 6        1400.000

# Create multiple columns
v12 <- rp_mutate(
  diamonds, 
  price_per_carat = price / carat,
  depth_pct = depth / 100
)
print(head(v12))
#>   carat       cut color clarity depth table price    x    y    z
#> 1  0.23     Ideal     E     SI2  61.5    55   326 3.95 3.98 2.43
#> 2  0.21   Premium     E     SI1  59.8    61   326 3.89 3.84 2.31
#> 3  0.23      Good     E     VS1  56.9    65   327 4.05 4.07 2.31
#> 4  0.29   Premium     I     VS2  62.4    58   334 4.20 4.23 2.63
#> 5  0.31      Good     J     SI2  63.3    58   335 4.34 4.35 2.75
#> 6  0.24 Very Good     J    VVS2  62.8    57   336 3.94 3.96 2.48
#>   price_per_carat depth_pct
#> 1        1417.391     0.615
#> 2        1552.381     0.598
#> 3        1421.739     0.569
#> 4        1151.724     0.624
#> 5        1080.645     0.633
#> 6        1400.000     0.628

# Remove one or more columns
v13 <- rp_mutate(diamonds, to_remove = c("table", "depth"))
print(head(v13))
#>   carat       cut color clarity price    x    y    z
#> 1  0.23     Ideal     E     SI2   326 3.95 3.98 2.43
#> 2  0.21   Premium     E     SI1   326 3.89 3.84 2.31
#> 3  0.23      Good     E     VS1   327 4.05 4.07 2.31
#> 4  0.29   Premium     I     VS2   334 4.20 4.23 2.63
#> 5  0.31      Good     J     SI2   335 4.34 4.35 2.75
#> 6  0.24 Very Good     J    VVS2   336 3.94 3.96 2.48
```


**5. rp_summarize():**
This verb collapses a data frame into a summary, often after grouping. It is similar to dplyr::summarise().
```{r}

# Summarize the entire data frame
v14 <- rp_summarize(diamonds, avg_price = mean(price), max_carat = max(carat))
print(v14)
#>    price carat
#> 1 3932.8   NaN
#> 2    NaN  5.01


# Group by one column (unquoted)
v15 <- rp_summarize(diamonds, avg_price = mean(price), .by = cut)
print(v15)
#>         cut avg_price
#> 1      Fair  4358.758
#> 2      Good  3928.864
#> 3 Very Good  3981.760
#> 4   Premium  4584.258
#> 5     Ideal  3457.542

# Group by multiple columns (unquoted)
v16 <- rp_summarize(
  diamonds, 
  avg_price = mean(price), 
  count = n(),
  .by = c(cut, color)
)
print(head(v16))
#>    cut color avg_price count
#> 1 Fair     D  4291.061   163
#> 2 Fair     E  3682.312   224
#> 3 Fair     F  3827.003   312
#> 4 Fair     G  4239.255   314
#> 5 Fair     H  5135.683   303
#> 6 Fair     I  4685.446   175

# Grouping also accepts quoted column names
v17 <- rp_summarize(
  diamonds, 
  avg_price = mean(price), 
  .by = c("cut", "color")
)
print(head(v17))
#>    cut color avg_price
#> 1 Fair     D  4291.061
#> 2 Fair     E  3682.312
#> 3 Fair     F  3827.003
#> 4 Fair     G  4239.255
#> 5 Fair     H  5135.683
#> 6 Fair     I  4685.446
```

**6. rp_calculate()– Apply Multiple Functions to Multiple Columns**
This verb is a powerful alternative to rp_summarize. It applies the same set of functions (e.g., "mean", "sd") to every selected column.
```{r}

# Apply two functions to two columns, grouped by 'cut'
v13 <- rp_calculate(
  diamonds,
  price, carat,
  the.functions = c("mean", "sd"),
  .by = cut
)
print(head(v13))
#>         cut price.mean price.std carat.mean carat.std
#> 1      Fair   4358.758  3560.387  1.0461366 0.5164043
#> 2      Good   3928.864  3681.590  0.8491847 0.4540544
#> 3 Very Good   3981.760  3935.862  0.8063814 0.4594354
#> 4   Premium   4584.258  4349.205  0.8919549 0.5152616
#> 5     Ideal   3457.542  3808.401  0.7028370 0.4328763
```

**7. rp_first_k_rows() and rp_last_k_rows() – First/Last K Rows per Group**
These verbs extract the first or last k rows from the whole data frame, or from each group when .by is provided.
```{r} 
# First 3 rows overall
v19 <- rp_first_k_rows(diamonds, k = 3)
print(v19)
#>   carat     cut color clarity depth table price    x    y    z
#> 1  0.23   Ideal     E     SI2  61.5    55   326 3.95 3.98 2.43
#> 2  0.21 Premium     E     SI1  59.8    61   326 3.89 3.84 2.31
#> 3  0.23    Good     E     VS1  56.9    65   327 4.05 4.07 2.31

# Last 2 rows per group (cut and clarity)
v20 <- rp_last_k_rows(diamonds, k = 2, .by = c(cut, clarity))
print(head(v20))
#>   carat     cut color clarity depth table price    x    y    z
#> 1  0.70    Fair     J    VVS1  67.6    54  1691 5.56 5.41 3.71
#> 2  0.50    Fair     D    VVS1  65.9    64  1792 4.92 5.03 3.28
#> 3  0.52    Fair     F      IF  64.6    58  2144 5.04 5.17 3.30
#> 4  0.47    Fair     D      IF  60.6    60  2211 5.09 4.98 3.05
#> 5  0.55    Good     F      IF  60.8    60  2266 5.26 5.36 3.23
#> 6  0.54 Premium     F      IF  61.9    60  2391 5.26 5.21 3.24

# Both quoted and unquoted group specifications work
v21 <- rp_first_k_rows(diamonds, k = 1, .by = c("cut", "color"))
print(v21)
#>    carat       cut color clarity depth table price    x    y    z
#> 1   0.23     Ideal     E     SI2  61.5  55.0   326 3.95 3.98 2.43
#> 2   0.21   Premium     E     SI1  59.8  61.0   326 3.89 3.84 2.31
#> 3   0.23      Good     E     VS1  56.9  65.0   327 4.05 4.07 2.31
#> 4   0.29   Premium     I     VS2  62.4  58.0   334 4.20 4.23 2.63
#> 5   0.31      Good     J     SI2  63.3  58.0   335 4.34 4.35 2.75
#> 6   0.24 Very Good     J    VVS2  62.8  57.0   336 3.94 3.96 2.48
#> 7   0.24 Very Good     I    VVS1  62.3  57.0   336 3.95 3.98 2.47
#> 8   0.26 Very Good     H     SI1  61.9  55.0   337 4.07 4.11 2.53
#> 9   0.22      Fair     E     VS2  65.1  61.0   337 3.87 3.78 2.49
#> 10  0.23     Ideal     J     VS1  62.8  56.0   340 3.93 3.90 2.46
#> 11  0.22   Premium     F     SI1  60.4  61.0   342 3.88 3.84 2.33
#> 12  0.30     Ideal     I     SI2  62.0  54.0   348 4.31 4.34 2.68
#> 13  0.30      Good     I     SI2  63.3  56.0   351 4.26 4.30 2.71
#> 14  0.23 Very Good     E     VS2  63.8  55.0   352 3.85 3.92 2.48
#> 15  0.23 Very Good     G    VVS2  60.4  58.0   354 3.97 4.01 2.41
#> 16  0.23 Very Good     D     VS2  60.5  61.0   357 3.96 3.97 2.40
#> 17  0.23 Very Good     F     VS1  60.9  57.0   357 3.96 3.99 2.42
#> 18  0.23      Good     F     VS1  58.2  59.0   402 4.06 4.08 2.37
#> 19  0.31      Good     H     SI1  64.0  54.0   402 4.29 4.31 2.75
#> 20  0.26      Good     D     VS2  65.2  56.0   403 3.99 4.02 2.61
#> 21  0.23     Ideal     G     VS1  61.9  54.0   404 3.93 3.95 2.44
#> 22  0.22   Premium     D     VS2  59.3  62.0   404 3.91 3.88 2.31
#> 23  0.30   Premium     J     SI2  59.3  61.0   405 4.43 4.38 2.61
#> 24  0.30     Ideal     D     SI1  62.5  57.0   552 4.29 4.32 2.69
#> 25  0.31   Premium     G     SI1  61.8  58.0   553 4.35 4.32 2.68
#> 26  0.30   Premium     H     SI1  62.9  59.0   554 4.28 4.24 2.68
#> 27  0.96      Fair     F     SI2  66.3  62.0  2759 6.27 5.95 4.07
#> 28  0.81     Ideal     F     SI2  58.8  57.0  2761 6.14 6.11 3.60
#> 29  0.91      Fair     H     SI2  64.4  57.0  2763 6.11 6.09 3.93
#> 30  0.77     Ideal     H     VS2  62.0  56.0  2763 5.89 5.86 3.64
#> 31  0.72      Good     G     VS2  59.7  60.5  2776 5.80 5.84 3.47
#> 32  0.84      Fair     G     SI1  55.1  67.0  2782 6.39 6.20 3.47
#> 33  1.05      Fair     J     SI2  65.8  59.0  2789 6.41 6.27 4.18
#> 34  0.90      Fair     I     SI1  67.3  59.0  2804 5.93 5.84 3.96
#> 35  0.75      Fair     D     SI2  64.6  57.0  2848 5.74 5.72 3.70

```

**8. rp_count() – Count Rows (Overall or by Group)**
This verb returns the number of rows in the data frame, optionally by groups.
```{r}
# Total row count
v22 <- rp_count(diamonds)
print(v22)
#>       n
#> 1 53940

# Count per group
v23 <- rp_count(diamonds, .by = cut)
print(v23)
#>         cut     n
#> 1      Fair  1610
#> 2      Good  4906
#> 3 Very Good 12082
#> 4   Premium 13791
#> 5     Ideal 21551

# Count per combination of multiple groups
v24 <- rp_count(diamonds, .by = c(cut, color))
print(head(v24))
#>    cut color   n
#> 1 Fair     D 163
#> 2 Fair     E 224
#> 3 Fair     F 312
#> 4 Fair     G 314
#> 5 Fair     H 303
#> 6 Fair     I 175
```

## Chaining Verbs with the Pipe
All rPandas functions are "pipe-friendly" and use .data as their first argument, allowing you to chain operations together using magrittr's %>% pipe (or the native R |>).

```{r}
# Load the pipe
v25 <- diamonds |>
  rp_filter(carat > 1 & color == "D") |>
  rp_mutate(price_per_carat = price / carat) |>
  rp_summarize(avg_ppc = mean(price_per_carat), .by = cut) |>
  rp_sort(desc(avg_ppc))

print(head(v25))
#>         cut  avg_ppc
#> 1     Ideal 7546.163
#> 2 Very Good 6789.316
#> 3   Premium 6548.397
#> 4      Good 5784.918
#> 5      Fair 5414.87
```

## Viewing the Python Code
A key feature of rPandas is its role as a learning and translation tool. Every user-facing verb has a return.as argument that lets you inspect the Python code it generates.

You can set return.as to:

**"result" (default):** Returns the final R data frame.

**"code":** Returns the generated Python command as a string.

**"all":** Returns a list containing both the result and the code.

```{r}
# See the code for a simple filter
rp_filter(diamonds, carat > 1 & price < 400, return.as = "code")
#> [1] "df.query('(carat > 1) and (price < 400)')"

# See the code for a mutate
rp_mutate(diamonds, ppc = price / carat, return.as = "code")
#> [1] "df.assign(ppc = lambda x: (x['price'] / x['carat']))"

# See the code for a complex summary
rp_summarize(
  diamonds,
  avg_price = mean(price), 
  count = n(),
  .by = c(cut, color),
  return.as = "code"
)
#> [1] "df.groupby(['cut', 'color'], as_index=False, observed=True).agg(avg_price = ('price', 'mean'), count = ('price', 'size'))"
```

## Customizing the Generated Code with table_name
By default, when you request return.as = "code", the generated Python code uses the placeholder rpandas_df_in as the DataFrame name. This is because the function only receives the 
data object, not its name. However, you can provide a custom name using the table_name argument (available in all verbs). This is especially useful when you want to copy‑paste the 
code into a Python script or notebook.
```{r}
# Default placeholder
rp_filter(diamonds, carat > 1, return.as = "code")
#> [1] "df.query('carat > 1')"

# With custom table name
rp_filter(diamonds, carat > 1, table_name = "diamonds", return.as = "code")
#> [1] "diamonds.query('carat > 1')"

```
The output changes from:
**"rpandas_df_in.query('(carat > 1)')"** to: **"diamonds.query('(carat > 1)')"**.
This works for all verbs – rp_select, rp_mutate, rp_summarize, etc. Simply pass table_name = "your_data_frame_name" as an argument.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_filter.R
\name{rp_filter}
\alias{rp_filter}
\title{Filter rows using pandas}
\usage{
rp_filter(.data, filter_expression, table_name = NULL, return.as = "result")
}
\arguments{
\item{.data}{An R data.frame or tibble.}

\item{filter_expression}{The filtering expression, written in R syntax.}

\item{table_name}{An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).}

\item{return.as}{What to return: "result", "code", or "all".}
}
\value{
A \code{data.frame} containing the filtered rows.
}
\description{
Filters a data frame using an R expression translated to pandas.
}
\examples{
\donttest{
if (reticulate::py_available(initialize = TRUE) &&
    reticulate::py_module_available("pandas")) {
      rp_filter(ggplot2::diamonds, carat > 1 & price < 4000)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{translate_groupby}
\alias{translate_groupby}
\title{Translate a .by argument into a pandas .groupby() string}
\usage{
translate_groupby(by_expr)
}
\arguments{
\item{by_expr}{An enquosured \code{.by} argument.}
}
\value{
A string for the \code{.groupby(..., as_index=False)} method,
or NULL if the argument is empty.
}
\description{
Translate a .by argument into a pandas .groupby() string
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/execute_pandas_statement.R
\name{execute_pandas_statement}
\alias{execute_pandas_statement}
\title{Executes a pandas command string on an R data frame.}
\usage{
execute_pandas_statement(
  r_df,
  py_command,
  table_name = table_name,
  return.as = "result"
)
}
\arguments{
\item{r_df}{An R data.frame.}

\item{py_command}{A character string of Python code using 'df' as a placeholder.}

\item{table_name}{An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).}

\item{return.as}{What to return: "result", "code", or "all".}
}
\value{
The result of the execution.
}
\description{
This is the core execution engine. It explicitly injects an R data frame
into the Python session, runs a command, retrieves the result, and cleans up.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_filter.R
\name{rp_first_k_rows}
\alias{rp_first_k_rows}
\title{Extract the first k rows of a data frame}
\usage{
rp_first_k_rows(.data, k, .by = NULL, table_name = NULL, return.as = "result")
}
\arguments{
\item{.data}{An R data frame (or tibble) to be processed.}

\item{k}{An integer specifying the number of rows to return. If \code{.by} is used,
returns up to k rows per group.}

\item{.by}{Optional grouping variables. Can be one or more unquoted column names.
When provided, the operation is performed on each group separately.}

\item{table_name}{An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).}

\item{return.as}{One of \code{"result"}, \code{"code"}, or \code{"all"}.}
}
\value{
Depending on \code{return.as}: a data frame, a character string, or a list.
}
\description{
This function returns the first k rows of the data frame. If grouping variables
are provided via \code{.by}, it returns the first k rows within each group.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_filter.R
\name{rp_count}
\alias{rp_count}
\title{Count rows in a data frame, optionally by groups}
\usage{
rp_count(.data, .by = NULL, table_name = NULL, return.as = "result")
}
\arguments{
\item{.data}{An R data frame (or tibble) to be processed.}

\item{.by}{Optional grouping variables. Can be one or more unquoted column names
(e.g., \code{cut} or \code{c(cut, color)}). When provided, counts are computed per group.}

\item{table_name}{An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).}

\item{return.as}{One of \code{"result"}, \code{"code"}, or \code{"all"}.}
}
\value{
A data frame with one column \code{"n"} (total row count) if \code{.by = NULL},
or a data frame with the grouping columns and a column \code{"n"} (per‑group counts).
}
\description{
This function returns the number of rows in a data frame. When grouping
variables are provided via \code{.by}, it returns the row counts for each group.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{translate_mutate}
\alias{translate_mutate}
\title{Translate R mutate expressions to pandas assign/drop strings}
\usage{
translate_mutate(to_remove = NULL, ...)
}
\arguments{
\item{to_remove}{Character vector of column names to drop.}

\item{...}{Named R expressions.}
}
\value{
A list with components assign_str and drop_str.
}
\description{
Translate R mutate expressions to pandas assign/drop strings
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{translate_filter}
\alias{translate_filter}
\title{Translate an R filter expression into a Python query string}
\usage{
translate_filter(expr)
}
\arguments{
\item{expr}{A bare R expression (e.g., \code{carat > 2 & cut == "Ideal"}).}
}
\value{
A character string of the translated Python query.
}
\description{
Capture a bare R expression and translate it to a Python-compatible string
suitable for use with \code{pandas.DataFrame.query()}.
}
\examples{
translate_filter(carat > 2 & cut == "Ideal")
# -> "(carat > 1) and (cut == 'Ideal')"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{translate_sort}
\alias{translate_sort}
\title{Translate captured sort expressions into Python .sort_values() arguments}
\usage{
translate_sort(...)
}
\arguments{
\item{...}{Bare column names or \code{desc(colname)} (captured by \code{rlang::enquos}).}
}
\value{
A list with two elements:
\itemize{
\item \verb{$by}: A string for the \code{by} argument (e.g., \code{['cut', 'price']})
\item \verb{$ascending}: A string for the \code{ascending} argument (e.g., \code{[True, False]})
}
}
\description{
Translate captured sort expressions into Python .sort_values() arguments
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_pandas_statement.R
\name{create_pandas_statement}
\alias{create_pandas_statement}
\title{Create a chained pandas command string}
\usage{
create_pandas_statement(
  df_name,
  filter_str = NULL,
  select_str = NULL,
  sort_by_str = NULL,
  sort_asc_str = NULL,
  assign_str = NULL,
  drop_str = NULL,
  groupby_str = NULL,
  agg_str = NULL,
  head_k = NULL,
  tail_k = NULL
)
}
\arguments{
\item{df_name}{A character string for the name of the pandas DataFrame.}

\item{filter_str}{A string for the \code{.query()} method.}

\item{select_str}{A string for column selection (e.g., \code{['col1', 'col2']}).}

\item{sort_by_str}{A string for the \code{by} argument of \code{.sort_values()}.}

\item{sort_asc_str}{A string for the \code{ascending} argument of \code{.sort_values()}.}

\item{assign_str}{A string for the \code{.assign()} method.}

\item{drop_str}{A string for the \code{.drop()} method.}

\item{groupby_str}{A string for the \code{.groupby()} method.}

\item{agg_str}{A string for the \code{.agg()} method.}

\item{head_k}{Integer for \code{.head(k)}.}

\item{tail_k}{Integer for \code{.tail(k)}.}
}
\value{
A character string of the complete, chained pandas command.
}
\description{
This internal function assembles a Python command string for pandas by
chaining together different data manipulation methods. It serves as the
central command generator for the package.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{\%notin\%}
\alias{\%notin\%}
\title{"Not In" Operator}
\usage{
x \%notin\% y
}
\arguments{
\item{x}{Vector of values to be matched.}

\item{y}{Vector of values to be matched against.}
}
\value{
A logical vector.
}
\description{
Provides the opposite of the standard R \code{\%in\%} operator.
}
\examples{
"a" \%notin\% c("b", "c")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_filter.R
\name{rp_select}
\alias{rp_select}
\title{Filtering columns}
\usage{
rp_select(.data, ..., table_name = NULL, return.as = "result")
}
\arguments{
\item{.data}{An R data.frame or tibble.}

\item{...}{The bare column names to select (e.g., \verb{carat, cut, price}).}

\item{table_name}{An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).}

\item{return.as}{What to return: "result", "code", or "all".}
}
\value{
A \code{data.frame} containing only the selected columns.
}
\description{
Selects specific columns from a data frame. It captures the bare column
names and translates the operation into a pandas selection command.
}
\examples{
\donttest{
if (reticulate::py_available(initialize = TRUE) &&
    reticulate::py_module_available("pandas")) {
  rp_select(ggplot2::diamonds, carat, cut, price)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{translate_summarize}
\alias{translate_summarize}
\title{Translate named R expressions for .agg()}
\usage{
translate_summarize(...)
}
\arguments{
\item{...}{Named R expressions (e.g., \code{avg_price = mean(price)}).}
}
\value{
A string for the \code{.agg()} method.
}
\description{
Translates R's \code{new = func(old)} syntax into pandas' named aggregation
syntax \verb{new = ('old', 'func')}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_setup.R
\name{rp_check_env}
\alias{rp_check_env}
\title{Check for rPandas dependencies and provide diagnostics}
\usage{
rp_check_env()
}
\value{
Invisibly returns \code{TRUE} if all checks pass, otherwise \code{FALSE}.
}
\description{
This function checks if the user's system is correctly configured with Python
and the pandas library. If dependencies are missing, it stops with a
detailed diagnostic report and actionable instructions (only in interactive sessions).
In non‑interactive contexts (e.g., CRAN checks), it issues a warning and returns FALSE.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_filter.R
\name{rp_calculate}
\alias{rp_calculate}
\title{Apply multiple summary functions to multiple columns}
\usage{
rp_calculate(
  .data,
  ...,
  the.functions,
  .by = NULL,
  table_name = NULL,
  return.as = "result"
)
}
\arguments{
\item{.data}{An R data.frame.}

\item{...}{Bare column names to summarize (e.g., \verb{price, carat}).}

\item{the.functions}{A character vector of R function names
(e.g., \code{c("mean", "sd")}). Supports "mean", "median", "sd",
"var", "min", "max", "sum".}

\item{.by}{A bare column name or \code{c(col1, col2)} to group by.}

\item{table_name}{An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).}

\item{return.as}{What to return: "result", "code", or "all".}
}
\value{
A \code{data.frame} with the summarized and grouped data.
}
\description{
Applies a list of summary functions to a list of columns, after
optionally grouping the data.
}
\examples{
\donttest{
if (reticulate::py_available(initialize = TRUE) &&
    reticulate::py_module_available("pandas")) {

  rp_calculate(
    ggplot2::diamonds,
    price, carat,
    the.functions = c("mean", "sd"),
    .by = cut
  )
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rPandas-package.R
\docType{package}
\name{rPandas-package}
\alias{rPandas-package}
\alias{rPandas}
\title{rPandas: A User-Friendly R Interface to Pandas}
\description{
This package provides a set of wrapper functions that allow R users to
interact with Python's pandas library using familiar R syntax.
}
\author{
\strong{Maintainer}: Akshat Maurya \email{codingmaster902@gmail.com}

Authors:
\itemize{
  \item Rihaan Satia
  \item David Shilane \email{david.shilane@columbia.edu}
}

}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_filter.R
\name{rp_mutate}
\alias{rp_mutate}
\title{Mutate (add/modify/remove) columns using pandas}
\usage{
rp_mutate(
  .data,
  to_remove = NULL,
  ...,
  table_name = NULL,
  return.as = "result"
)
}
\arguments{
\item{.data}{An R data frame.}

\item{to_remove}{A character vector of column names to remove.}

\item{...}{Named expressions for new/modified columns.}

\item{table_name}{An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).}

\item{return.as}{Either "result", "code", or "all".}
}
\value{
A data frame or list depending on return.as.
}
\description{
Mutate (add/modify/remove) columns using pandas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{translate_assign_recursive}
\alias{translate_assign_recursive}
\title{Recursively translate an R expression for a pandas .assign() lambda}
\usage{
translate_assign_recursive(expr_body)
}
\arguments{
\item{expr_body}{A language object (call, symbol, or atomic).}
}
\value{
A character string of the translated Python expression.
}
\description{
Recursively translate an R expression for a pandas .assign() lambda
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{translate_select}
\alias{translate_select}
\title{Translate captured column names into a Python list string}
\usage{
translate_select(...)
}
\arguments{
\item{...}{Bare column names (captured by \code{rlang::enquos}).}
}
\value{
A character string formatted as a Python list.
}
\description{
Translate captured column names into a Python list string
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_filter.R
\name{rp_summarize}
\alias{rp_summarize}
\title{Summarize data using pandas}
\usage{
rp_summarize(.data, ..., .by = NULL, table_name = NULL, return.as = "result")
}
\arguments{
\item{.data}{An R data.frame or tibble.}

\item{...}{Named summary expressions (e.g., \code{avg_price = mean(price)}).
Supports \code{mean}, \code{median}, \code{sd}, \code{var}, \code{min}, \code{max}, \code{sum}, and \code{n()}.}

\item{.by}{A bare column name or \code{c(col1, col2)} to group by.}

\item{table_name}{An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).}

\item{return.as}{What to return: "result", "code", or "all".}
}
\value{
A \code{data.frame} with the summarized and grouped data.
}
\description{
Aggregates a data frame by one or more groups, applying summary functions.
It translates R's \code{dplyr::summarise} syntax into a pandas \verb{.groupby().agg()}
command.
}
\examples{
\donttest{
if (reticulate::py_available(initialize = TRUE) &&
    reticulate::py_module_available("pandas")) {
  
  # Summarize by one group
  rp_summarize(ggplot2::diamonds, 
               avg_price = mean(price), 
               .by = cut)
  
  # Summarize by multiple groups and multiple functions
  rp_summarize(ggplot2::diamonds, 
               avg_price = mean(price), 
               count = n(),
               .by = c(cut, color))
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_filter.R
\name{rp_sort}
\alias{rp_sort}
\title{Sort rows of a data frame using pandas}
\usage{
rp_sort(.data, ..., table_name = NULL, return.as = "result")
}
\arguments{
\item{.data}{An R data.frame or tibble.}

\item{...}{Bare column names to sort by. Use \code{desc(colname)} to sort
in descending order (e.g., \verb{cut, desc(price)}).}

\item{table_name}{An optional character string. If provided, the generated Python code will replace the internal dataframe name with this string (e.g., \code{"diamonds.query(...)"}). This is useful for seeing the exact, copy-pasteable Python code. Defaults to \code{NULL} (uses \code{"df"}).}

\item{return.as}{What to return: "result", "code", or "all".}
}
\value{
A \code{data.frame} sorted by the specified columns.
}
\description{
Sorts a data frame by one or more columns. It translates the R expressions
into a pandas \code{.sort_values()} command and executes it.
}
\examples{
\donttest{
if (reticulate::py_available(initialize = TRUE) &&
    reticulate::py_module_available("pandas")) {
  
  # Sort by cut (ascending) and price (descending)
  rp_sort(ggplot2::diamonds, cut, desc(price))
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{translate_calculate}
\alias{translate_calculate}
\title{Translate R function/column names into a pandas agg dictionary}
\usage{
translate_calculate(variable_exprs, function_names)
}
\arguments{
\item{variable_exprs}{A list of enquosured variable names.}

\item{function_names}{A character vector of R function names.}
}
\value{
A string for the \code{.agg()} method (e.g., \code{.agg({'col1': ['mean', 'std']})}).
}
\description{
Translates R's \code{the.variables} and \code{the.functions} into pandas'
dictionary-based \code{.agg()} syntax.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_engine.R
\name{translate_filter_recursive}
\alias{translate_filter_recursive}
\title{Recursive helper to translate R expressions}
\usage{
translate_filter_recursive(expr_body, env)
}
\arguments{
\item{expr_body}{A language object (call, symbol, or atomic).}
}
\value{
A character string.
}
\description{
Recursive helper to translate R expressions
}
\keyword{internal}
